from makeitparallel import Communicator, initialization, parallel

# Create a communicator object
communicator = Communicator()


# Here we'll create a function that will generate a dummy file to do the demonstration.
# We'll also use the initialization decorator because we want it only to happen in the root process.
@initialization
def create_example_list(number_of_elements):
    return list(range(number_of_elements))


# Read the file just in the root process:
test_list = create_example_list(communicator.comm_size)

if test_list is not None:
    # Check what do the different processes have
    print("I'm %s and I have" % (str(communicator.rank)), test_list)
else:
    # Check what do the different processes have
    print("I'm %s and I don't have anything" % (str(communicator.rank)))


# Now we can use parallel to define a function that will implicitly distribute the elements of the list
# before doing something with it

@parallel
def distributed_work_with_list(input_list):
    print("I'm %i and I've done work with:" % communicator.rank, input_list)
    return sum(input_list)


# Synchronize processes for output clarity
communicator.sync()

# Use the function
returned = distributed_work_with_list(test_list)


# The returned values will be merged again in the root process.
"""
Running with 8 tasks the output will look like:

--------------------------------------------------------------------------
--------------------------------------------------------------------------
I'm 6 and I don't have anything
I'm 7 and I don't have anything
I'm 3 and I don't have anything
I'm 0 and I have [0, 1, 2, 3, 4, 5, 6, 7]
I'm 4 and I don't have anything
I'm 1 and I don't have anything
I'm 5 and I don't have anything
I'm 2 and I don't have anything
I'm 0 and I've done work with: [0]
I'm 1 and I've done work with: [1]
I'm 4 and I've done work with: [4]
I'm 5 and I've done work with: [5]
I'm 6 and I've done work with: [6]
I'm 2 and I've done work with: [2]
I'm 3 and I've done work with: [3]
I'm 7 and I've done work with: [7]

--------------------------------------------------------------------------
"""
